__all__ = ['parse_json_data_to_scopus_df',]

    
def _built_date(dic, key):
    # Standard library imports
    import calendar
    
    # Local library imports
    from ScopusApyJson.json_utils import check_not_none
    from ScopusApyJson.json_utils import check_true_to_append
    from ScopusApyJson.json_utils import get_json_key_value
        
    dateitems = get_json_key_value(dic, key)
    if check_not_none(dateitems):
        dateitems_list = []  
        dateitems_list = check_true_to_append(dateitems, '@day', dateitems_list)
        item_num  = 0
        dateitems_list = check_true_to_append(dateitems, '@month', dateitems_list)
        item_num += 1
        month_num = int(dateitems_list[item_num])
        dateitems_list[item_num] = calendar.month_name[month_num]
        dateitems_list = check_true_to_append(dateitems, '@year', dateitems_list)
        date = " ".join(dateitems_list)

        return date


def _parse_source_info(json_data, article_dic):
    '''Parse the field "source" under the top 
    "field asbtracts-retrieval-response/item/bibrecord/head".
    '''
    
    # Local library imports
    from ScopusApyJson.json_utils import check_not_none
    from ScopusApyJson.json_utils import check_true_to_append
    from ScopusApyJson.json_utils import check_true_to_set
    from ScopusApyJson.json_utils import get_json_key_value
    
    source_dict = get_json_key_value(json_data, "source")
    if check_not_none(source_dict):
        article_dic['Year']                     = get_json_key_value(source_dict, "year")
        article_dic['Source title']             = get_json_key_value(source_dict, 'sourcetitle')
        article_dic['Abbreviated Source Title'] = get_json_key_value(source_dict, 'sourcetitle-abbrev')
        article_dic['CODEN']                    = get_json_key_value(source_dict, 'codencode')

        # Parsing issn
        issn_info = get_json_key_value(source_dict, 'issn')
        if check_not_none(issn_info):
            if not isinstance(issn_info, list):
                article_dic['ISSN'] = get_json_key_value(issn_info, '$')
            else:    
                for issn_dict in issn_info:
                    issn_type = get_json_key_value(issn_dict, '@type')
                    if issn_type == "print": article_dic['ISSN'] = get_json_key_value(issn_dict, '$')

        #Parsing isbn if available  
        isbn_info = get_json_key_value(source_dict, 'isbn')
        if check_not_none(isbn_info):
            if not isinstance(isbn_info, list):
                article_dic['ISBN'] = get_json_key_value(isbn_info, '$')            
            else:
                for isbn_dict in isbn_info:
                    isbn_type = get_json_key_value(isbn_dict, '@type')
                    if isbn_type == "print": article_dic['ISBN'] = get_json_key_value(isbn_dict, '$')

        # Parsing conference info if available
        conference_info = get_json_key_value(source_dict, 'additional-srcinfo')
        if check_not_none(conference_info):
            conferenceinfo = get_json_key_value(conference_info, 'conferenceinfo')
            if check_not_none(conferenceinfo):
                confevent = get_json_key_value(conferenceinfo, 'confevent')
                if check_not_none(confevent):
                    article_dic['Conference name'] = get_json_key_value(confevent, 'confname')                
                    article_dic['Conference code'] = get_json_key_value(confevent, 'confcode')
                    conflocation = get_json_key_value(confevent, 'conflocation')
                    if check_not_none(conflocation): 
                        article_dic['Conference location'] = get_json_key_value(conflocation, 'city')
                    confdates = get_json_key_value(confevent, 'confdate')
                    if check_not_none(confdates):
                        start_date = _built_date(confdates, 'startdate')
                        end_date   = _built_date(confdates, 'enddate')                   
                        article_dic['Conference date'] = start_date + " through " + end_date        

        # Parsing volisspag if available          
        article_dic = check_true_to_set(source_dict, 'article-number', article_dic, 'Art. No.')
        volisspag   = get_json_key_value(source_dict, "volisspag")
        if check_not_none(volisspag):
            voliss      = get_json_key_value(volisspag, "voliss")
            article_dic = check_true_to_set(voliss, '@volume', article_dic, 'Volume')
            article_dic = check_true_to_set(voliss, '@issue', article_dic, 'Issue')
            pagerange   = get_json_key_value(volisspag, "pagerange")
            if check_not_none(pagerange): 
                article_dic['Page start'] = get_json_key_value(pagerange, '@first')
                article_dic['Page end']   = get_json_key_value(pagerange, '@last')
                article_dic['Page count'] = str(int(article_dic['Page end']) - int(article_dic['Page start']))

        # Parsing publication stage
        stage_info  = get_json_key_value(source_dict, 'publicationdate')
        if "month" in stage_info.keys(): 
            article_dic['Publication Stage'] = "Final" 
        else:
            article_dic['Publication Stage'] = "Article in press"

        
def _parse_citation_info(json_data, article_dic):
    '''Parse the field "citation-info" under the top field 
    "asbtracts-retrieval-response/item/bibrecord/head".
    '''
    # Local library imports
    from ScopusApyJson.json_utils import check_not_none
    from ScopusApyJson.json_utils import get_json_key_value
    
    citation_info = get_json_key_value(json_data, 'citation-info')
    if check_not_none(citation_info):
        language_dict = get_json_key_value(citation_info, 'citation-language')
        if check_not_none(language_dict): 
            article_dic['Language of Original Document'] = get_json_key_value(language_dict, '@language')

    
def _parse_ordered_authors(json_data, article_dic):
    '''Parse the field "author" under the top field 
    "astracts-retrieval-response/authors".
    The field "author" is a dict if there is only 
    one author otherwise it is a list of dict.
    '''
    
    # Local library imports
    from ScopusApyJson.json_utils import check_not_none
    from ScopusApyJson.json_utils import get_json_key_value
    
    authors            = []
    authors_ids        = []
    authors_full_names = []
    
    auths_field = get_json_key_value(json_data, "authors")
    if check_not_none(auths_field):
        auths_group = get_json_key_value(auths_field, "author")
        if check_not_none(auths_group):
            if not isinstance(auths_group, list) : auths_group = [auths_group]
            for auths in auths_group:                
                auth_preferred_name = get_json_key_value(auths, 'preferred-name')
                if check_not_none(auth_preferred_name):
                    auth_id         = get_json_key_value(auths, '@auid')
                    auth_name       = get_json_key_value(auth_preferred_name, 'ce:indexed-name')
                    auth_surname    = get_json_key_value(auth_preferred_name, 'ce:surname')
                    auth_given_name = get_json_key_value(auth_preferred_name, 'ce:given-name')
                    
                    authors_ids.append(auth_id)
                    authors.append(auth_name)                
                    authors_full_names.append(f'{auth_surname}, {auth_given_name} ({auth_id})')

            article_dic['Authors']           = '; '.join(authors)
            article_dic['Author(s) ID']      = '; '.join(authors_ids)
            article_dic['Author full names'] = '; '.join(authors_full_names)
        
    
def _parse_authors_affiliations(json_data, article_dic):
    '''Parse the field "author-group" under the top field 
    "abstracts-retrieval-response/item/bibrecord/head".
    The field "author-group" is a dict if there is only one author, 
    otherwise it is a list of dict .
    '''
    
    # Standard library imports
    from collections import defaultdict
    
    # Local library imports
    from ScopusApyJson.json_utils import check_not_none
    from ScopusApyJson.json_utils import check_true_to_append
    from ScopusApyJson.json_utils import get_json_key_value
    
    affiliations_group = get_json_key_value(json_data, 'author-group')
    if check_not_none(affiliations_group):
    
        authors_with_affiliations_dict = defaultdict(list)
        authors_with_affiliations_list = []
        affiliations_list              = []
        ordered_authors                = article_dic['Authors'].split('; ')
        
        if not isinstance(affiliations_group, list) : affiliations_group = [affiliations_group]

        for affiliation_dict in affiliations_group:
            affiliation = get_json_key_value(affiliation_dict, 'affiliation')

            # Building affiliation full address
            organizations_list = get_json_key_value(affiliation, 'organization')
            if not isinstance(organizations_list, list) : organizations_list = [organizations_list]
            address_items_list = []
            for organization_dict in organizations_list:
                address_items_list.append(get_json_key_value(organization_dict, '$'))        
            address_items_list = check_true_to_append(affiliation, 'address-part', address_items_list)
            address_items_list = check_true_to_append(affiliation, 'city', address_items_list)
            address_items_list = check_true_to_append(affiliation, 'postal-code', address_items_list)
            address_items_list = check_true_to_append(affiliation, 'country', address_items_list)
            affiliation_address = ', '.join(address_items_list)

            # Appending "affiliation_address" to "affiliations_list"
            affiliations_list.append(affiliation_address)

            # Appending "affiliation_address" to "authors_with_affiliations_dict" for each author of the "affiliation_authors_list"
            affiliation_authors_list = get_json_key_value(affiliation_dict, 'author')
            if not isinstance(affiliation_authors_list, list) : affiliation_authors_list = [affiliation_authors_list]
            for author in affiliation_authors_list:
                author_preferred_name = get_json_key_value(author, 'preferred-name')
                author_name           = get_json_key_value(author_preferred_name, 'ce:indexed-name')
                authors_with_affiliations_dict[author_name].append(affiliation_address)

        # Ordering the "authors_with_affiliations_list" in the order of the "ordered_authors"
        for author in ordered_authors:
            author_affiliations_list = authors_with_affiliations_dict[author]
            authors_with_affiliations_list.append(f"{author}, {', '.join(author_affiliations_list)}")        

        article_dic['Authors with affiliations'] = '; '.join(authors_with_affiliations_list)
        article_dic['Affiliations']              = '; '.join(affiliations_list)

    
def _parse_correspondence_address(json_data, article_dic):
    '''Parse the field "correspondence" under the top field 
    "abstracts-retrieval-response/item/bibrecord/head".
    The field "correspondence" is a dict if there is only one corresponding person, 
    otherwise it is a list of dict.
    '''
    # Local library imports
    from ScopusApyJson.json_utils import check_not_none
    from ScopusApyJson.json_utils import check_true_to_append
    from ScopusApyJson.json_utils import get_json_key_value    

    correspondence_list = get_json_key_value(json_data, 'correspondence')
    if check_not_none(correspondence_list):
        person_address_list = []
        if not isinstance(correspondence_list, list) : correspondence_list = [correspondence_list]
        for correspondence_dict in correspondence_list:        
            correspondence_person_dict = get_json_key_value(correspondence_dict, 'person')
            if check_not_none(correspondence_person_dict):
                correspondence_person = get_json_key_value(correspondence_person_dict, 'ce:indexed-name')

            # Building affiliation full address
            affiliation = get_json_key_value(correspondence_dict, 'affiliation')
            if check_not_none(affiliation):
                organizations_list = get_json_key_value(affiliation, 'organization')
                if not isinstance(organizations_list, list) : organizations_list = [organizations_list]
                address_items_list = []
                for organization_dict in organizations_list:
                    address_items_list.append(get_json_key_value(organization_dict, '$'))
                address_items_list = check_true_to_append(affiliation, 'address-part', address_items_list)
                address_items_list = check_true_to_append(affiliation, 'city', address_items_list)
                address_items_list = check_true_to_append(affiliation, 'postal-code', address_items_list)
                address_items_list = check_true_to_append(affiliation, 'country', address_items_list)
                correspondence_address = ', '.join(address_items_list)

            person_address_list.append(correspondence_person + "; " + correspondence_address)

        article_dic['Correspondence Address'] = '; '.join(person_address_list)

    
def _parse_references(json_data, article_dic):
    '''Parse the field "bibliography" under the top field 
    "abstracts-retrieval-response/item/bibrecord/tail".
    The field "bibliography" is a dict keyyed by "$" if it is not None.
    '''
    # Local library imports
    from ScopusApyJson.json_utils import check_not_none
    from ScopusApyJson.json_utils import get_json_key_value   
    
    tail = get_json_key_value(json_data, 'tail')
    if check_not_none(tail):
        bibliography     = get_json_key_value(tail, 'bibliography')
        if check_not_none(bibliography):
            references_list  = get_json_key_value(bibliography, 'reference')
            if check_not_none(references_list):
                if not isinstance(references_list, list): references_list = [references_list]
                ref_text_list = []
                for ref_dict in references_list:
                    if check_not_none(ref_dict):
                        ref_info = get_json_key_value(ref_dict, 'ref-info')                        
                        if check_not_none(ref_info):
                            ref_item_list = []
                            ref_authors = get_json_key_value(ref_info, 'ref-authors')
                            if check_not_none(ref_authors):
                                author_dict = get_json_key_value(ref_authors, 'author')
                                if check_not_none(author_dict):
                                    et_al = ""
                                    if isinstance(author_dict, list): 
                                        author_dict = author_dict[0]
                                        et_al = " et al."
                                    author_name = get_json_key_value(author_dict, 'ce:indexed-name')
                                    if check_not_none(author_name):
                                        authors = author_name + et_al                
                                        ref_item_list.append(authors)

                            ref_title = get_json_key_value(ref_info, 'ref-title')
                            if check_not_none(ref_title):
                                title = get_json_key_value(ref_title, 'ref-titletext')
                                if check_not_none(title): ref_item_list.append(title)

                            source = get_json_key_value(ref_info, 'ref-sourcetitle')
                            if check_not_none(source): ref_item_list.append(source)

                            ref_volisspag = get_json_key_value(ref_info, 'ref-volisspag')
                            if check_not_none(ref_volisspag):    
                                voliss = get_json_key_value(ref_volisspag, 'voliss')
                                if check_not_none(voliss):
                                    volume = get_json_key_value(voliss, '@volume')
                                    issue  = get_json_key_value(voliss, '@issue')
                                    if check_not_none(volume): ref_item_list.append(volume)        
                                    if check_not_none(issue): ref_item_list.append(issue)
                                pagerange = get_json_key_value(ref_volisspag, 'pagerange')
                                if check_not_none(pagerange):
                                    page_first = get_json_key_value(pagerange, '@first')
                                    page_last  = get_json_key_value(pagerange, '@last')
                                    if check_not_none(page_first) and check_not_none(page_last): 
                                        ref_item_list.append('pp. ' + page_first + '-' + page_last)

                            ref_publicationyear = get_json_key_value(ref_info, 'ref-publicationyear')
                            if check_not_none(ref_publicationyear):
                                year = get_json_key_value(ref_publicationyear, '@first')
                                if check_not_none(year): ref_item_list.append("(" + year + ")")

                            refd_itemidlist = get_json_key_value(ref_info, 'refd-itemidlist')
                            if check_not_none(refd_itemidlist):
                                itemid_list = get_json_key_value(refd_itemidlist, 'itemid')
                                if check_not_none(itemid_list):
                                    if not isinstance(itemid_list, list): itemid_list = [itemid_list]
                                    doi = None
                                    for itemid_dict in itemid_list:
                                        itemid_type = get_json_key_value(itemid_dict, '@idtype') 
                                        if itemid_type == "DOI": doi = get_json_key_value(itemid_dict, '$')
                                    if check_not_none(doi): ref_item_list.append(doi)

                            ref_text = get_json_key_value(ref_info, 'ref-text')
                            if check_not_none(ref_text): ref_item_list.append(ref_text)

                            full_ref = ', '.join(ref_item_list)
                            ref_text_list.append(full_ref)
                        article_dic['References'] = '; '.join(ref_text_list)            

        
def _parse_index_keywords(json_data, article_dic):
    '''Parse the field "idxterms" under the top field "abstracts-retrieval-response".
    The field "idxterms" is a dict keyyed by "$" if it is not None.
    '''
    # Local library imports
    from ScopusApyJson.json_utils import check_not_none
    from ScopusApyJson.json_utils import get_json_key_value
    
    idxterms = get_json_key_value(json_data, 'idxterms')
    if check_not_none(idxterms):
        idxterms_list = get_json_key_value(idxterms, 'mainterm')
        if not isinstance(idxterms_list, list): idxterms_list = [idxterms_list]
        article_dic['Index Keywords'] = '; '.join([x['$'] for x in idxterms_list])

        
def _parse_author_keywords(json_data, article_dic):
    '''Parse the field "authkeywords" under the top field "abstracts-retrieval-response".
    The field "authkeywords" is a dict keyyed by "$" if it is not None.
    '''
    # Local library imports
    from ScopusApyJson.json_utils import check_not_none
    from ScopusApyJson.json_utils import get_json_key_value
    
    author_keywords = get_json_key_value(json_data, 'authkeywords')
    if check_not_none(author_keywords):
        author_keywords_list = get_json_key_value(author_keywords, 'author-keyword')
        if check_not_none(author_keywords_list):
            if not isinstance(author_keywords_list, list): author_keywords_list = [author_keywords_list]
            article_dic['Author Keywords'] = '; '.join([x['$'] for x in author_keywords_list])   


def _parse_coredata(json_data, article_dic):
    '''Parse the field "coredata" under the top field "abstracts-retrieval-response".
    '''
    # Local library imports
    from ScopusApyJson.json_utils import check_not_none
    from ScopusApyJson.json_utils import get_json_key_value
    
    coredata_dict = get_json_key_value(json_data, 'coredata')
    if check_not_none(coredata_dict):
        article_dic['Title']         = get_json_key_value(coredata_dict, 'dc:title')
        article_dic['DOI']           = get_json_key_value(coredata_dict, 'prism:doi')
        article_dic['EID']           = get_json_key_value(coredata_dict, 'eid')
        article_dic['Document Type'] = get_json_key_value(coredata_dict, 'subtypeDescription')
        article_dic['PubMed ID']     = get_json_key_value(coredata_dict, 'pubmed-id')
        article_dic['Publisher']     = get_json_key_value(coredata_dict, 'dc:publisher')
        article_dic['Cited by']      = get_json_key_value(coredata_dict, 'citedby-count')
        article_dic['Abstract']      = get_json_key_value(coredata_dict, 'dc:description')

        # Parsing open access
        openaccess = get_json_key_value(coredata_dict, 'openaccess')
        if openaccess == "2" : 
            article_dic['Open Access'] = "All Open Access; Green Open Access"
        if openaccess == "1" : 
            article_dic['Open Access'] = "All Open Access; Green Open Access; Gold Open Access (Hybrid?)"

        # Parsing link
        link_info = get_json_key_value(coredata_dict, 'link')
        if check_not_none(link_info):
            if isinstance(link_info, list):        
                for link_dict in link_info:
                    link_rel = get_json_key_value(link_dict, '@rel')
                    if link_rel == "scopus": article_dic['Link'] = get_json_key_value(link_dict, '@href')
            else:
                link_dict =  link_info
                article_dic['Link'] = get_json_key_value(link_dict, '@href')


def _make_json_data_dict(json_data, article_dic):
    
    # Setting default values (not in json_data)
    article_dic['Source'] = "Scopus"
    
    # Parsing json data
    _parse_ordered_authors(json_data, article_dic)
    _parse_authors_affiliations(json_data, article_dic)
    _parse_correspondence_address(json_data, article_dic)
    _parse_source_info(json_data, article_dic)
    _parse_citation_info(json_data, article_dic)    
    _parse_references(json_data, article_dic)
    _parse_index_keywords(json_data, article_dic)
    _parse_author_keywords(json_data, article_dic)
    _parse_coredata(json_data, article_dic)
    
    return article_dic


def parse_json_data_to_scopus_df(json_data):
    '''The function `parse_json_data_to_scopus_df` parses 
    the hierarchical dict of the data "json_data". Then it builds
    the dataframe which colums are specified by the global 
    "SELECTED_SCOPUS_COLUMNS_NAMES".
    
    Args:
        json_data (dict): The hierarchical dict built by the function 
        `get_doi_json_data_from_api`.
        
    Returns:
        (pandas.core.frame.DataFrame): The dataframe resulting 
        from the hierarchical dict parsing.
    '''
    # Standard library imports
    import json as json
    
    # 3rd party imports
    import pandas as pd
    
    # Globals imports 
    from ScopusApyJson.GLOBALS import PARSED_SCOPUS_COLUMNS_NAMES
    from ScopusApyJson.GLOBALS import SELECTED_SCOPUS_COLUMNS_NAMES
    
    article_dic = {}
    for key in set(PARSED_SCOPUS_COLUMNS_NAMES):
        article_dic[key] = None 

    article_dic = _make_json_data_dict(json_data, article_dic)               
    
    scopus_df = pd.DataFrame.from_dict([article_dic])
    scopus_df = scopus_df[SELECTED_SCOPUS_COLUMNS_NAMES]
    
    return scopus_df



__all__ = ['build_scopus_df_from_api',
          ]

def build_scopus_df_from_api(doi_list):
    '''The function `build_scopus_df_from_api` gets, for each of the DOI 
    in the DOI list "doi_list", the hierarchical dict of the data returned by 
    the function 'get_doi_json_data_from_api'. Then it builts the dataframes resulting 
    from the hierarchical dicts parsing using the function `parse_json_data_to_scopus_df`.
    Finally, it concatenates these dataframe to a single dataframe.
    
    Args:
        doi_list (list): The list of DOI (str) for the Scopus API request.
        
    Returns:
        (pandas.core.frame.DataFrame): The concatenation of the dataframes resulting 
    from the hierarchical dicts parsing for each DOI of the list "doi_list".
    
    '''
    # 3rd party imports
    import pandas as pd
    
    # Local library imports
    from ScopusApyJson.GLOBALS import SELECTED_SCOPUS_COLUMNS_NAMES
    from ScopusApyJson.api_manager import get_doi_json_data_from_api
    from ScopusApyJson.json_parser import parse_json_data_to_scopus_df
    
    if not isinstance(doi_list, list): doi_list = [doi_list]
    scopus_df_list = []
    api_scopus_df = pd.DataFrame(columns = SELECTED_SCOPUS_COLUMNS_NAMES)
    for idx, doi in enumerate(doi_list):        
        api_json_data, request_status = get_doi_json_data_from_api(doi)        
        if request_status == "Empty":
            print(f'DOI {doi} not found in Scopus database')
        elif request_status == "False":
            print('Authentication failed: please check availability of authentication keys')
        elif request_status == "Timeout":
            print('Request timeout: please check web access')
        else:
            print(f'Resquest successful for DOI {doi}')           
            scopus_df = parse_json_data_to_scopus_df(api_json_data)
            scopus_df_list.append(scopus_df)
            api_scopus_df = pd.concat(scopus_df_list, axis = 0)
    
    return api_scopus_df



"""
A template of the config json file named "api_scopus_config.json" is stored in the ".ScopusApyJson/CONFIG" folder.
The user's "api_scopus_config.json" should be located in the user's "~/AppData/Roaming/ScopusApyJson" folder 
and the user is invited to fill this file with its scopus api authentication keys.    
If the user's "api_scopus_config.json" file exits in the "~/AppData/Roaming/ScopusApyJson" folder, it will be used. 
Otherwise, a copy of the template file of the ".ScopusApyJson/CONFIG" folder will be automatically created 
in the user's "~/AppData/Roaming/ScopusApyJson" folder in order to be filled with the user's scopus api authentication keys.
"""

__all__ = ['API_CONFIG_DICT',
           'API_CONFIG_PATH',
           'ELS_LINK',
           'PARSED_SCOPUS_COLUMNS_NAMES',
           'SELECTED_SCOPUS_COLUMNS_NAMES',
          ]

# Header of the query for the Scopus API
ELS_LINK = "https://api.elsevier.com/content/abstract/" 

# Internal functions to build the Globals
def _get_config_dir():

    """
    Returns a parent directory path
    where persistent application data can be stored.

    # linux: ~/.local/share
    # macOS: ~/Library/Application Support
    # windows: C:/Users/<USER>/AppData/Roaming
    adapted from : https://stackoverflow.com/questions/19078969/python-getting-appdata-folder-in-a-cross-platform-way
    """
    # Standard library imports
    import sys
    from pathlib import Path
    
    home = Path.home()

    if sys.platform == 'win32':
        return home / Path('AppData/Roaming')
    elif sys.platform == 'linux':
        return home / Path('.local/share')
    elif sys.platform == 'darwin':
        return home / Path('Library/Application Support')

def _dump_json(file_path, json_dict):
    # Standard library imports
    import json
    
    with open(file_path, 'w') as file:
        json.dump(json_dict, file, indent=4)
    
def _check_api_keys(API_CONFIG_DICT):
    # Standard library imports
    import warnings
      
    dict_apikey    = API_CONFIG_DICT["apikey"]
    dict_insttoken = API_CONFIG_DICT["insttoken"]
    if (dict_apikey in  ["PAST_APIKEY_HERE", ""]) or (dict_insttoken in ["PAST_INSTTOKEN_HERE", ""]):
        message  = "Authentication keys are not yet defined (required).\n"
        warnings.warn(message)
        API_CONFIG_DICT["apikey"]    = input("Enter your authentication key (obtained from http://dev.elsevier.com):")
        API_CONFIG_DICT["insttoken"] = input("your institution token provided by Elsevier support staff:")
        _dump_json(API_CONFIG_PATH, API_CONFIG_DICT)     

def _config_ScopusApyJson_dict(json_file_name):
    # Standard library imports
    import json
    import os.path
    from pathlib import Path
    
    # Reads the default json_file_name config file
    pck_config_path = Path(__file__).parent / Path('CONFIG') / Path(json_file_name)
    pck_config_date = os.path.getmtime(pck_config_path)
    with open(pck_config_path) as file:
        json_config_dict = json.load(file)
        
    # Sets the json_config_dict according to the status of the local config file        
    local_config_dir_path  = _get_config_dir() / Path('ScopusApyJson')
    local_config_file_path = local_config_dir_path  / Path(json_file_name)
    
    if os.path.exists(local_config_file_path):
        # Local json_file_name config file exists
        local_config_date = os.path.getmtime(local_config_file_path)
        if local_config_date > pck_config_date:
            # Local config file is more recent than package one
            # thus json_config_dict is defined by the local config file
            with open(local_config_file_path) as file:
                json_config_dict = json.load(file)
        else:
            # Local config file is less recent than package one
            # thus package config file overwrite the local config file 
            # and json_config_dict is kept at default values
            _dump_json(local_config_file_path, json_config_dict)
    else:
        # Local json_file_name config file does not exist
        # thus package config file is used to create a local config file
        # to be filled by the user
        # and json_config_dict is kept at default values
        if not os.path.exists(_get_config_dir() / Path('ScopusApyJson')):
            os.makedirs(_get_config_dir() / Path('ScopusApyJson'))
        _dump_json(local_config_file_path, json_config_dict)      
    
    return json_config_dict, local_config_file_path

# Getting the authentication key and the institution token through the json file 'api_scopus_config.json'
# stored in the folder ".ScopusApyJson/CONFIG"
API_CONFIG_DICT, API_CONFIG_PATH = _config_ScopusApyJson_dict('api_scopus_config.json')
_check_api_keys(API_CONFIG_DICT)

# Getting the names of the selected scopus columns by the user through the json file "scopus_col_names.json" 
# stored in the folder ""~/AppData/Roaming/ScopusApyJson" of vthe user
scopus_column_names_dict, _   = _config_ScopusApyJson_dict('scopus_col_names.json')
PARSED_SCOPUS_COLUMNS_NAMES   = list(scopus_column_names_dict.keys())[1:]
SELECTED_SCOPUS_COLUMNS_NAMES = [k for k,v in scopus_column_names_dict.items() if v][1:] 
  


__author__  = 'BiblioAnalysis team'
__license__ = 'MIT'


from ScopusApyJson.GLOBALS import *
from ScopusApyJson.json_utils import *
from ScopusApyJson.json_parser import *
from ScopusApyJson.api_manager import *
from ScopusApyJson.main import *


#!/usr/bin/env python

# Standard library imports
from setuptools import setup, find_packages
from os import path

# Reading the content of the "README.md" file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Reading the content of the "requirements.txt" file
with open(path.join(this_directory, 'requirements.txt'), encoding='utf-8') as f:
    install_requires = f.read().strip().split('\n')

# Setting the setup parameters
setup(name = 'ScopusApyJson',
      version = '1.1.0',
      description = 'Python modules for parsing the response to a Scopus API request',
      long_description = long_description,
      long_description_content_type = 'text/markdown',
      include_package_data = True,
      license = 'MIT',
      classifiers = [
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.9',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Operating System :: OS Independent',
        'Intended Audience :: Science/Research'
        ],
      keywords = 'Metadata parsing, Scopus request, API management',
      install_requires = install_requires,
      entry_points = {
        'console_scripts': [
                            'cli_doi = ScopusApyJson.CLI.cli:cli_doi',
                            'cli_json = ScopusApyJson.CLI.cli:cli_json'
                            ],
         },
      author = 'BiblioAnalysis team',
      author_email = 'francois.bertin7@wanadoo.fr, amal.chabli@orange.fr',
      url = 'https://github.com/TickyWill/ScopusApyJson',
      packages = find_packages(), 
      )



__all__ = ['get_doi_json_data_from_api',]

def _set_els_doi_api(MyScopusKey, MyInstKey, doi):
    """The internal function `_set_els_doi_api` sets, for the DOI 'doi', 
    the query 'els_api' according to the Scopus API usage 
    which header is given by the global 'ELS_LINK'.
    
    Args:
        MyScopusKey (str): The user's authentication key.
        MyInstKey (str): The user's institution token.
        doi (str): The publication DOI for which the Scopus API will provide information. 
        
    Returns:
        (str): The query for the passed DOI according to the scopus api usage.
        
    """ 
    # Globals imports
    from ScopusApyJson.GLOBALS import ELS_LINK
    
    # Setting the query  
    query_header = ELS_LINK
    query = doi + '?'

    # Building the HAL API
    els_api = query_header \
            + query \
            + '&apikey='    + MyScopusKey \
            + '&insttoken=' + MyInstKey \
            + '&httpAccept=application/json'
    
    return els_api


def _get_json_from_api(doi, api_config_dict):
    """The internal function `_get_json_from_api` gets, for the DOI 'doi', 
    the response to the query 'els_api' built using the internal function `_set_els_doi_api`.
    It passes to this function the user's authentication key 'MyScopusKey' and the user's 
    institutional token 'MyInstKey' given by the dict 'api_config_dict'. 
    It also increments the number of requests performed by the user. The number is updated 
    in the dict 'api_config_dict' at key 'api_uses_nb'.
    
    Args:
        doi (str): The publication DOI for which the Scopus API will provide data.
        api_config_dict (dict): The dict wich values are the user's authentication key, the user's 
        institutional token and the number of requests performed. 
        
    Returns:
        (tup): The tup composed by the hierarchical-dict response
        to the query and the updated 'api_config_dict' dict.

    """    
   
    # 3rd party library imports
    import requests
    from requests.exceptions import Timeout
    
    # Setting client authentication keys
    MyScopusKey = api_config_dict["apikey"]
    MyInstKey   = api_config_dict["insttoken"]
    api_uses_nb = api_config_dict['api_uses_nb']
    if (MyScopusKey in  ["PAST_APIKEY_HERE", ""]) or (MyInstKey in ["PAST_INSTTOKEN_HERE", ""]):
        response_status = "Wrong authentication"

    # Setting Elsevier API
    els_api = _set_els_doi_api(MyScopusKey, MyInstKey, doi)
    
    # Initializing parameters
    response_dict = None

    # Get the request response
    try:
        response = requests.get(els_api, timeout = 10)   
    except Timeout:
        response_status = "Timeout"
    else:
        if response == False: # response.status_code <200
            response_status = "False"
        else:
            if response.status_code in [204, 404]:
                response_status = "Empty"
            else:
                response_status = "True"
                response_dict = response.json()
                
        # Updating api_uses_nb in config_dict
        api_config_dict["api_uses_nb"] = api_uses_nb + 1
    
    return (response_dict, api_config_dict, response_status)


def _update_api_config_json(API_CONFIG_PATH, API_CONFIG_DICT):
    # Standard library imports
    import json
    
    with open(API_CONFIG_PATH, 'w') as f:
        json.dump(API_CONFIG_DICT, f, indent = 4)
        
        
def get_doi_json_data_from_api(doi):
    """The function `get_doi_json_data_from_api` gets, for the DOI 'doi', 
    the json-serialized response to the Scopus API request using 
    the internal function `_get_json_from_api`.
    It passes to this function the user's dict 'API_CONFIG_DICT'. 
    It also updates the API configuration json file with the modified 
    dict 'API_CONFIG_DICT' returned by this function.
    
    Args:
        doi (str): The publication DOI for which the Scopus API will provide data.
        
    Returns:
        (dict): The hierarchical dict of the data returned by the internal function '_get_json_from_api'.
        
    """ 
    
    # Globals imports
    from ScopusApyJson.GLOBALS import API_CONFIG_DICT
    from ScopusApyJson.GLOBALS import API_CONFIG_PATH
    
    # Getting api json data
    doi_json_data, API_CONFIG_DICT, request_status = _get_json_from_api(doi, API_CONFIG_DICT)
    
    # Updatting api config json with number of requests
    _update_api_config_json(API_CONFIG_PATH, API_CONFIG_DICT)
    
    return (doi_json_data, request_status)


__all__ = ['check_not_none',
           'check_true_to_append',
           'check_true_to_set',
           'get_json_key_value',
          ]

def get_json_key_value(dic, key):
    '''Gets the value at key "key" of a hierarchical dict "dic".
    It checks the possible multiple occurences of the key.
    It returns value only in the case of a single occurence of the key.
    
    Args:
        dic (dict): The hierarchical dict.
        key (str): The key which value is searched in the dict "dic".
        
    Returns:
        (str, list or dict): The value at key "key" in the hierarchical 
        dict "dic" for a single occurence of the key, 
        otherwise "multiple_key" message or None if the dict is empty.
    
    '''
    if dic:
        hit   = []
        stack = list(zip(dic.keys(), dic.values()))
        while stack :
            k,v = stack.pop()
            if k == key:
                hit.append(v)
            if isinstance(v, dict):
                stack.extend(list(zip(v.keys(), v.values()))) 

        if len(hit) == 1: return hit[0]
        if len(hit) > 1 : return "multiple_key"
    else: return None


def check_not_none(value):
    '''Checks if the parameter "value" is not None or not equal to "None".
    
    Args:
        value (str): The parameter to check.
        
    Returns:
        (bool): True if value is not None or not equal to "None" (default: False).
        
    '''
    status = False
    if value and value != "None": 
        status = True 
    return status
        
    
def check_true_to_append(dic, key, items_list):
    '''Appends to the list "items_list", the value at key "key" of a hierarchical dict "dic" 
    in case the return of the function `check_not_none` is True. 
    
    Args:
        dic (dict): The hierarchical dict.
        key (str): The key which value is searched in the dict "dic".
        items_list (list): The list to append.
        
    Returns:
        (list): The list appended with the value at key "key" of a hierarchical 
        dict "dic" (default: unchanged list).
        
    '''
    value = get_json_key_value(dic, key)
    if check_not_none(value): items_list.append(value)
    return items_list


def check_true_to_set(dic1, key1, dic2, key2):
    '''Sets the value at key "key2" of a dict "dic2" 
    to the value "value" returned by the function `get_json_key_value` 
    applied to the hierarchical dict "dic1" and key "key1" 
    in case the return of the function `check_not_none` 
    applied to value "value" is True. 
    
    Args:
        dic1 (dict): The hierarchical dict.
        key1 (str): The key which value is searched in the dict "dic1".
        dic2 (dict): The dict to be updated.
        key2 (str): The key at which the dict "dic2" will be updated.
        
    Returns:
        (dict): The dict "dic2" updated at key "key2" with the value at key "key1" of the hierarchical 
        dict "dic1" (default: unchanged dict "dic2").
        
    '''    
    value = get_json_key_value(dic1, key1)
    if check_not_none(value): dic2[key2] = value
    return dic2



def cli_doi():

    # Standard library imports
    from argparse import ArgumentParser, Namespace
    from pathlib  import Path  
    
    # 3rd party imports
    import pandas as pd
    
    # Local library imports
    import ScopusApyJson as saj
    from ScopusApyJson.main import build_scopus_df_from_api    
    
    parser = ArgumentParser()
    parser.usage = '''Usage cli_doi -d <doi1 doi2 ...>  -o <results folder path>
    1- Get the json data through scopus api request for each DOI of a list of DOIs. 
    2- Saves the results in a csv file in the folder <results folder path> (default: your homedir).
    '''
    
    parser.add_argument('-d',
                        '--doi',
                        help='doi to parse',
                        type=str,
                        nargs='+',
                        required=True)

    parser.add_argument('-o', '--output',
                        nargs='?', 
                        help='output path for the results')
    
    args : Namespace = parser.parse_args()
    
    if args.output is None:
        out_path = Path.home()
        print(f'Default output directory : {out_path}')
    else:
        out_path = args.output
        print(f'Option and command-line argument given: "-o {out_path}"')
    
    doi_list = args.doi
    api_scopus_df = build_scopus_df_from_api(doi_list)
    output_file = out_path / Path('scopus_result.csv')
    api_scopus_df.to_csv(output_file,
                         header = True,
                         index = False,
                         sep = ',')
    print(f"The results have been saved in the file: {output_file}")
    

def cli_json():    
    from argparse import ArgumentParser, Namespace

    parser = ArgumentParser()
    parser.add_argument('file',help='json file to parse', type=str)
    args : Namespace = parser.parse_args()
    print(args.file)
    
